/*
  File name        : queue.c
  Date of creation : 10/6/2009
  Version          : 1.0
  Copyright        : Soft'n'design
  Author           : Laurent Mazet <mazet@softndesign.org>

  Description      : This file contains tests for queue thread.

  History          :
  - initial version
*/

/* depend: */
/* cflags: */
/* linker: -lmcore -lpthread -lm */

#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#include <snd/queue.h>

int pseudo_random (unsigned int seed)
{

    int i;

    /* Pseudo random sequence initialisation */
    unsigned int x1 = 1;
    unsigned int x2 = seed;
    unsigned int x3 = 1 - seed;

    /* Pseudo random sequence initial scrambling */
    for (i = 0; i < 1000; i++) {
        unsigned int state;

        state = ((x1 >> 3) ^ x1) & 1;
        x1 = (state << 30) | (x1 >> 1);

        state = ((x2 >> 3) ^ (x2 >> 2) ^ (x2 >> 1) ^ x2) & 1;
        x2 = (state << 30) | (x2 >> 1);

        state = ((x3 >> 4) ^ (x2 >> 2) ^ x2) & 1;
        x3 = (state << 30) | (x2 >> 1);
    }

    int r = (x1 ^ x2 ^ x3);

    return r;
}

int randn (void)
{

    int i = 0;
    float x, y;

    int r = 0;
    for (i = 0; i < 10; i++) {

        do
            /* generate a signal which is average random in [0,1] */
            x = (float) rand () / ((float)RAND_MAX + 1);

        while (x == 0);     /* x should not be zero */

        /* generate another signal which is average random in [0,1] */
        y = (float) rand () / ((float)RAND_MAX + 1);

        /* generate noise which average is 0, and variance is sigma */
        float z = sqrtf (-2. * logf (x)) * cosf (2 * M_PI * y);

        r = z * (1 << 31);
    }

    return r;
}

typedef struct {
    int *buffer;
    int length;
} data_t;

void huge_process_init (void)
{
    printf ("init done\n");
}

int huge_process (void *_param)
{

    int i;
    data_t *param = (data_t *)_param;

    /* new seed */
    int seed = randn();

    /* fill buffer */
    for (i = 0; i < param->length; i++) {
        param->buffer[i] = pseudo_random (seed);
        seed = param->buffer[i];
    }

    return 0;
}

int main (int argc, char *argv[])
{

    int i, j, rc;

    int nb_loops = 10;
    int nb_blocks = 30;
    int length = 10;

    /* process arguments */
    if (argc > 1) {
        printf ("usage: %s\n", argv[0]);
        exit (1);
    }

    /* init structure */
    data_t *d = (data_t *) calloc (nb_blocks, sizeof (data_t));
    for (i = 0; i < nb_blocks; i++) {
        d[i].buffer = (int *) calloc (length, sizeof (int));
        d[i].length = length;
    }

    /* create queue */
    queue_task_t *queue = create_queue_task ("process", 1000, 70, 2, "0+1+2+3:2", huge_process_init);

    for (j = 0; j < nb_loops; j++) {

        /* fill queue */
        queue_sub_task_info_t *queue_huge_process = queue_init (queue);

        for (i = 0; i < nb_blocks; i++) {
            rc = queue_load (queue, queue_huge_process, huge_process, d + i);
            if (rc == 0)
                printf ("error %d for load %d\n", rc, i);
        }

        /* queue processing */
        rc = queue_end (queue, queue_huge_process);
        if (rc == 0)
            printf ("error %d on queue ending\n", rc);

        /* results */
        printf ("results:");
        for (i = 0; i < nb_blocks; i++)
            printf (" 0x%08x", d[i].buffer[d[i].length - 1]);
        printf ("\n");

        /* statistics */
        queue_stat (queue, 1);
        printf ("\n");
    }

    /* statistics */
    queue_stat (queue, 0);

    /* cleaning */
    for (i = 0; i < nb_blocks; i++)
        free (d[i].buffer);
    free (d);

    return 0;
}

/* vi:set tabstop=4 expandtab shiftwidth=4: this line set vi mode */
